# Short Message Service Gateway

## Functions

- [SmsDisableTrace](#smsdisabletrace-function)
- [SmsEnableTrace](#smsenabletrace-function)
- [SmsEnumProviders](#smsenumproviders-function)
- [SmsGetErrorString](#smsgeterrorstring-function)
- [SmsGetFirstProvider](#smsgetfirstprovider-function)
- [SmsGetGateway](#smsgetgateway-function)
- [SmsGetLastError](#smsgetlasterror-function)
- [SmsGetNextProvider](#smsgetnextprovider-function)
- [SmsGetProvider](#smsgetprovider-function)
- [SmsInitialize](#smsinitialize-function)
- [SmsSendMessage](#smssendmessage-function)
- [SmsSetLastError](#smssetlasterror-function)
- [SmsUninitialize](#smsuninitialize-function)
## Structures

- [SMSGATEWAY](#smsgateway-structure)
- [SMSMESSAGE](#smsmessage-structure)
- [SMSPROVIDER](#smsprovider-structure)
- [SMSSERVICE](#smsservice-structure)

---

## SmsDisableTrace Function

Disable logging of network function calls

### Prototype

```c
BOOL WINAPI SmsDisableTrace();
```

### Parameters

| Name | Type | Description |
|------|------|-------------|

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstxtv11.lib`

### See Also

- [SmsEnableTrace](#smsenabletrace-function)

---

## SmsEnableTrace Function

Enable logging of network function calls to a text file

### Prototype

```c
BOOL WINAPI SmsEnableTrace(
    LPCTSTR lpszTraceFile,
    DWORD dwTraceFlags
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszTraceFile` | `LPCTSTR` | A pointer to a string which specifies the name of the trace log file. If this parameter is NULL or an empty string, the default file name cstrace.log is used. The file is stored in the directory specified by the TEMP environment variable, if it is defined; otherwise, the current working directory is used. |
| `dwTraceFlags` | `DWORD` | An unsigned integer that specifies one or more options. This parameter is constructed by using a bitwise operator with any of the following values: |

### Options

- **dwTraceFlags** Parameter

    - `TRACE_INFO`: All function calls are written to the trace file. This is the default value.
    - `TRACE_ERROR`: Only those function calls which fail are recorded in the trace file.
    - `TRACE_WARNING`: Only those function calls which fail or return values which indicate a warning are recorded in the trace file.
    - `TRACE_HEXDUMP`: All functions calls are written to the trace file, plus all the data that is sent or received is displayed, in both ASCII and hexadecimal format.
    - `TRACE_PROCESS`: All function calls in the current process are logged, rather than only those functions in the current thread. This option is useful for multithreaded applications that are using worker threads.

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. A return value of zero indicates an error. Failure typically indicates that the tracing library cstrcv11.dll cannot be loaded on the local system.

### Remarks

- When trace logging is enabled, the file is opened, appended to and closed for each socket function call. This makes it possible for an application to append its own logging information, however care should be taken to ensure that the file is closed before the next network operation is performed. To limit the size of the log files, enable and disable logging only around those sections of code that you wish to trace.
- Note that the TRACE_HEXDUMP trace level generates very large log files since it includes all of the data exchanged between your application and the server.
- Trace function logging is managed on a per-thread basis, not for each client handle. This means that all SocketTools libraries and components share the same settings in the current thread. If you are using multiple SocketTools libraries or components in your application, you only need to enable logging once.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstxtv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SmsDisableTrace](#smsdisabletrace-function)

---

## SmsEnumProviders Function

Enumerate the available wireless service providers

### Prototype

```c
INT WINAPI SmsEnumProviders(
    LPSMSPROVIDER lpProviders,
    INT nMaxProviders,
    DWORD dwReserved
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpProviders` | `LPSMSPROVIDER` | A pointer to an array of SMSPROVIDER structures that will be populated with information about each service provider. If this parameter is NULL, the function will return the number of service providers that are known. |
| `nMaxProviders` | `INT` | An integer value which specifies the maximum number of service providers that may be enumerated by this function. If this parameter is zero, the function will return the number of service providers that are known. If the lpProviders parameter is NULL, this value should be zero. |
| `dwReserved` | `DWORD` | A reserved parameter. This value should always be zero. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the number of known service providers. If the function fails, the return value is SMS_ERROR. To get extended error information, call SmsGetLastError. If the lpProviders parameter is not NULL and the nMaxProviders parameter indicates the array is not large enough to store all of the provider information, this function will fail with an error indicating that the buffer is too small.

### Remarks

- The SmsEnumProviders function is used to enumerate all of the supported service providers, populating an array of SMSPROVIDER structures that contains information about each provider, such as their name, domain, region of the country they service and the maximum message size they will accept. Typically this would be used to update a user interface control such as a listbox or drop-down combobox, enabling a user to select a preferred service provider.
- Because some programming languages may not support arrays of structures in the same way that C/C++ does, the SmsGetFirstProvider and SmsGetNextProvider functions offer an alternative way to easily enumerate the available service providers.
- To obtain information about a single service provider, use the SmsGetProvider function.

### Example

```cpp
SMSPROVIDER smsProviders[MAXPROVIDERS];
INT nProviders;

nProviders = SmsEnumProviders(smsProviders, MAXPROVIDERS, 0);
if (nProviders == SMS_ERROR)
{
	DWORD dwError = SmsGetLastError();
	_tprintf(_T("An error has occurred, error code %d\n"), (INT)LOWORD(dwError));
}
else
{
	for (INT nIndex = 0; nIndex < nProviders; nIndex++)
		_tprintf(_T("%s\n"), smsProviders[nIndex].szName);

	_tprintf(_T("%d providers returned\n"), nProviders);
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstxtv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SmsGetFirstProvider](#smsgetfirstprovider-function)
- [SmsGetGateway](#smsgetgateway-function)
- [SmsGetNextProvider](#smsgetnextprovider-function)
- [SmsGetProvider](#smsgetprovider-function)
- [SMSPROVIDER](#smsprovider-function)

---

## SmsGetErrorString Function

Return a description for the specified error code

### Prototype

```c
INT WINAPI SmsGetErrorString(
    DWORD dwErrorCode,
    LPTSTR lpszDescription,
    INT cbDescription
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `dwErrorCode` | `DWORD` | The last-error code for which a description is returned. |
| `lpszDescription` | `LPTSTR` | A pointer to the buffer that will contain a description of the specified error code. This buffer should be at least 128 characters in length. |
| `cbDescription` | `INT` | The maximum number of characters that may be copied into the description buffer. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is the length of the description string. If the function fails, the return value is zero, meaning that no description exists for the specified error code. Typically this indicates that the error code passed to the function is invalid.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstxtv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SmsGetLastError](#smsgetlasterror-function)
- [SmsSetLastError](#smssetlasterror-function)

---

## SmsGetFirstProvider Function

Return information about the first supported wireless service provider

### Prototype

```c
BOOL WINAPI SmsGetFirstProvider(
    LPSMSPROVIDER lpProvider,
    LPDWORD lpdwToken
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpProvider` | `LPSMSPROVIDER` | A pointer to an SMSPROVIDER structure that will contain information about the service provider. This parameter cannot be NULL. |
| `lpdwToken` | `LPDWORD` | A pointer to an unsigned integer value that is used when enumerating the service providers. The value will be initialized by this function and the modified by subsequent calls to the SmsGetNextProvider function. This parameter cannot be NULL. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call SmsGetLastError.

### Remarks

- The SmsGetFirstProvider function is used in conjunction with SmsGetNextProvider to enumerate all of the supported wireless providers available to the client. These two functions can be used as an alternative to the SmsEnumProviders function, which may not be easily used with some programming languages because it populates an array of data structures rather than a single structure. The data that is returned is identical, the only difference is the method by which the service providers are enumerated.
- The lpdwToken parameter is used to maintain context between multiple calls to the SmsGetNextProvider function. It should be treated as an opaque value and never modified directly by the application.

### Example

```cpp
SMSPROVIDER smsProvider;
DWORD dwToken;
BOOL bResult;

bResult = SmsGetFirstProvider(&smsProvider, &dwToken);
while (bResult)
{
	pListBox->AddString(smsProvider.szName);
	bResult = SmsGetNextProvider(&smsProvider, &dwToken);
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstxtv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SmsEnumProviders](#smsenumproviders-function)
- [SmsGetGateway](#smsgetgateway-function)
- [SmsGetNextProvider](#smsgetnextprovider-function)
- [SMSPROVIDER](#smsprovider-function)

---

## SmsGetGateway Function

Return information about the gateway server for the specified phone number

### Prototype

```c
INT WINAPI SmsGetGateway(
    LPCTSTR lpszPhoneNumber,
    LPCTSTR lpszProvider,
    LPSMSGATEWAY lpGateway
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszPhoneNumber` | `LPCTSTR` | A pointer to a string which specifies the telephone number that you wish to obtain information about. Any whitespace, punctuation or other non-numeric characters in the string will be ignored. This parameter cannot be NULL. |
| `lpszProvider` | `LPCTSTR` | A pointer to a string which specifies the preferred service provider for this telephone number. If the preferred service provider is unknown, this parameter can be NULL and the default provider will be selected. |
| `lpGateway` | `LPSMSGATEWAY` | A pointer to an SMSGATEWAY structure that will contain information about the text message gateway when the function returns. This includes information such as the name of the provider, the server that will accept text messages for this phone number, and the recipient address that should be used. This parameter cannot be NULL. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is SMS_ERROR. To get extended error information, call SmsGetLastError.

### Remarks

- The SmsGetGateway function returns information about the service provider and mail gateway for a specific phone number, and can be used to determine if a given phone number is assigned to a mobile device capable of receiving text messages. This is done by sending an query to a server that will check the phone number against a database of known providers and the phone numbers that have been allocated for wireless devices. If the phone number is valid, information will be returned about the provider that is responsible for that number along with information about its text message gateway service.
- If the lpszProvider parameter is not NULL, this will identify a preferred provider for the phone number specified. In the United States and Canada, most wireless common carriers are required to provide wireless number portability (WNP) which allows a customer to continue to use their current phone number even if they switch to another service provider. This can result in a situation where a specific phone number is shown as allocated to one provider, but in actuality that user has switched to a different provider. For example, a user may have originally purchased a phone and service with AT&T and then later switched to Verizon, but decided to keep their phone number. In this case, if Verizon was not specified as the preferred provider, the library would attempt to send the message to the AT&T gateway, since that was the original provider who allocated the phone number.
- For most applications, the correct way to handle the situation in which a user may have switched to a different service provider is to allow them to select an alternate service provider in your user interface. For example, you could display a drop-down list of available service providers, populated using the SmsEnumProviders function. If they select a preferred provider, then you would pass that value to this function. If they do not, then specify a NULL pointer and the default provider will be selected.
- This function sends an HTTP query to the server api.sockettools.com to obtain information about the phone number and wireless service provider. This requires that the local system can establish a standard network connection over port 80. If the client cannot connect to the server, the function will fail and an appropriate error will be returned. The server imposes a limit on the maximum number of connections that can be established and the maximum number of requests that can be issued per minute. If this function is called multiple times over a short period, the library may also force the application to block briefly. Server responses are cached per session, so calling this function multiple times using the same phone number will not increase the request count.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstxtv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SmsEnumProviders](#smsenumproviders-function)
- [SmsGetProvider](#smsgetprovider-function)
- [SMSGATEWAY](#smsgateway-function)

---

## SmsGetLastError Function

Return the last error code

### Prototype

```c
DWORD WINAPI SmsGetLastError();
```

### Parameters

| Name | Type | Description |
|------|------|-------------|

### Return Value

- **Type**: `DWORD`
- The return value is the last error code value for the current thread. Functions set this value by calling the SmsSetLastError function. The Return Value section of each reference page notes the conditions under which the function sets the last error code.

### Remarks

- Error codes are unsigned 32-bit values which are private to each calling thread. You should call the SmsGetLastError function immediately when a function's return value indicates that an error has occurred. That is because some functions call SmsSetLastError(0) when they succeed, clearing the error code set by the most recently failed function.
- Most functions will set the last error code value when they fail; a few functions set it when they succeed. Function failure is typically indicated by a return value such as FALSE, NULL, INVALID_CHANNEL or SMS_ERROR. Those functions which call SmsSetLastError when they succeed are noted on the function reference page.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstxtv11.lib`

### See Also

- [SmsGetErrorString](#smsgeterrorstring-function)
- [SmsSetLastError](#smssetlasterror-function)

---

## SmsGetNextProvider Function

Return information about the next supported wireless service provider

### Prototype

```c
BOOL WINAPI SmsGetNextProvider(
    LPSMSPROVIDER lpProvider,
    LPDWORD lpdwToken
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpProvider` | `LPSMSPROVIDER` | A pointer to an SMSPROVIDER structure that will contain information about the service provider. This parameter cannot be NULL. |
| `lpdwToken` | `LPDWORD` | A pointer to an unsigned integer value that is used when enumerating the service providers. The value will be initialized by the SmsGetFirstProvider function and the modified by subsequent calls to this function. This parameter cannot be NULL. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the last service provider has been enumerated or the function fails, the return value is zero. To get extended error information, call SmsGetLastError.

### Remarks

- The SmsGetNextProvider function is used in conjunction with SmsGetFirstProvider to enumerate all of the supported wireless providers available to the client. These two functions can be used as an alternative to the SmsEnumProviders function, which may not be easily used with some programming languages because it populates an array of data structures rather than a single structure. The data that is returned is identical, the only difference is the method by which the service providers are enumerated.
- The lpdwToken parameter is used to maintain context between multiple calls to this function. It should be treated as an opaque value and never modified directly by the application.

### Example

```cpp
SMSPROVIDER smsProvider;
DWORD dwToken;
BOOL bResult;

bResult = SmsGetFirstProvider(&smsProvider, &dwToken);
while (bResult)
{
	pListBox->AddString(smsProvider.szName);
	bResult = SmsGetNextProvider(&smsProvider, &dwToken);
}
```

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstxtv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SmsEnumProviders](#smsenumproviders-function)
- [SmsGetGateway](#smsgetgateway-function)
- [SmsGetNextProvider](#smsgetnextprovider-function)
- [SMSPROVIDER](#smsprovider-function)

---

## SmsGetProvider Function

Return information about the wireless service provider for the specified phone number

### Prototype

```c
INT WINAPI SmsGetProvider(
    LPCTSTR lpszPhoneNumber,
    LPSMSPROVIDER lpProvider
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszPhoneNumber` | `LPCTSTR` | A pointer to a string which specifies the telephone number that you wish to obtain information about. Any whitespace, punctuation or other non-numeric characters in the string will be ignored. This parameter cannot be NULL. |
| `lpProvider` | `LPSMSPROVIDER` | A pointer to an SMSPROVIDER structure that will contain information about the service provider for the specified phone number. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is SMS_ERROR. To get extended error information, call SmsGetLastError.

### Remarks

- The SmsGetProvider function returns information about the service provider associated with a phone number. This is done by sending an query to a server that will check the phone number against a database of known providers and the phone numbers that have been allocated for wireless devices. If the phone number is valid, information will be returned about the provider that is responsible for that number.
- Because most wireless carriers in the United States and Canada must provide for wireless number portability, there is the possibility that the provider information returned may no longer correspond to the telephone number. It is recommended that you provide your end-user with the ability to specify an alternate preferred provider to use when sending the text message. For more information, refer to the SmsGetGateway function.
- This function sends an HTTP query to the server api.sockettools.com to obtain information about the wireless service provider. This requires that the local system can establish a standard network connection over port 80. If the client cannot connect to the server, the function will fail and an appropriate error will be returned. The server imposes a limit on the maximum number of connections that can be established and the maximum number of requests that can be issued per minute. If this function is called multiple times over a short period, the library may also force the application to block briefly. Server responses are cached per session, so calling this function multiple times using the same phone number will not increase the request count.
- For a list of all supported wireless service providers, use the SmsEnumProviders function.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstxtv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SmsEnumProviders](#smsenumproviders-function)
- [SmsGetGateway](#smsgetgateway-function)
- [SMSPROVIDER](#smsprovider-function)

---

## SmsInitialize Function

Initialize the library and validate the specified license key at runtime

### Prototype

```c
BOOL WINAPI SmsInitialize(
    LPCTSTR lpszLicenseKey,
    LPINITDATA lpData
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpszLicenseKey` | `LPCTSTR` | Pointer to a string that specifies the runtime license key to be validated. If this parameter is NULL, the library will validate the development license installed on the local system. |
| `lpData` | `LPINITDATA` | Pointer to an INITDATA data structure. This parameter may be NULL if the initialization data for the library is not required. |

### Return Value

- **Type**: `BOOL`
- If the function succeeds, the return value is non-zero. If the function fails, the return value is zero. To get extended error information, call SmsGetLastError. All other client functions will fail until a license key has been successfully validated.

### Remarks

- This must be the first function that an application calls before using any of the other functions in the library. When a NULL license key is specified, the library will only function on the development system. Before redistributing the application to an end-user, you must insure that this function is called with a valid license key.
- If the lpData argument is specified, it must point to an INITDATA structure which has been initialized by setting the dwSize member to the size of the structure. All other structure members should be set to zero. If the function is successful, then the INITDATA structure will be filled with identifying information about the library.
- Although it is only required that SmsInitialize be called once for the current process, it may be called multiple times; however, each call must be matched by a corresponding call to SmsUninitialize.
- This function dynamically loads other system libraries and allocates thread local storage. If you are calling the functions in this library from within another DLL, it is important that you do not call the SmsInitialize or SmsUninitialize functions from the DllMain function because it can result in deadlocks or access violation errors. If the DLL is linked with the C runtime library (CRT), it will automatically call the constructors and destructors for static and global C++ objects and has the same restrictions.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstxtv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SmsSendMessage](#smssendmessage-function)
- [SmsUninitialize](#smsuninitialize-function)

---

## SmsSendMessage Function

Send a text message to the specified mobile device

### Prototype

```c
INT WINAPI SmsSendMessage(
    LPSMSSERVICE lpService,
    LPSMSMESSAGE lpMessage,
    DWORD dwReserved
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `lpService` | `LPSMSSERVICE` | A pointer to an SMSSERVICE structure that identifies the messaging service that will be used to send the text message. The default service sends the message through the mail server gateway for the wireless service provider associated with the recipient's phone number. This parameter cannot be NULL. |
| `lpMessage` | `LPSMSMESSAGE` | A pointer to an SMSMESSAGE structure that contains information about the message to be sent, including the sender, the recipient and the text message itself. This parameter cannot be NULL. |
| `dwReserved` | `DWORD` | A reserved parameter. This value should always be zero. |

### Return Value

- **Type**: `INT`
- If the function succeeds, the return value is zero. If the function fails, the return value is SMS_ERROR. To get extended error information, call SmsGetLastError.

### Remarks

- The SmsSendMessage function is used to send a text message to a mobile device. This API is designed to support multiple methods of sending text messages, with the default method sending the message through a server gateway established by the wireless service provider.
- This message service sends the message through the wireless service provider's mail gateway using the SMTP protocol. However, it is important to note that many of these gateways will not accept messages from a client that is connected to them using a residential Internet service provider. If the application is being run on a system that uses a residential provider, that service provider may also block outbound connections to all mail servers other than their own. These anti-spam measures typically require that most end-user applications specify a relay mail server rather than submitting the message directly to the wireless provider's gateway.
- Because most wireless carriers in the United States and Canada must provide for wireless number portability, there is the possibility that the provider information returned may no longer correspond to the telephone number. It is recommended that you provide your end-user with the ability to specify an alternate preferred provider to use when sending the text message. For more information, refer to the SmsGetGateway function.
- This service also sends an HTTP query to the server api.sockettools.com to obtain information about the phone number and wireless service provider. This requires that the local system can establish a standard network connection over port 80. If the client cannot connect to the server, the function will fail and an appropriate error will be returned. The server imposes a limit on the maximum number of connections that can be established and the maximum number of requests that can be issued per minute. If this function is called multiple times over a short period, the library may also force the application to block briefly. Server responses are cached per session, so calling this function multiple times using the same phone number will not increase the request count.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstxtv11.lib`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SmsGetGateway](#smsgetgateway-function)
- [SmsGetProvider](#smsgetprovider-function)
- [SMSMESSAGE](#smsmessage-function)
- [SMSSERVICE](#smsservice-function)

---

## SmsSetLastError Function

Set the last error code

### Prototype

```c
VOID WINAPI SmsSetLastError(
    DWORD dwErrorCode
);
```

### Parameters

| Name | Type | Description |
|------|------|-------------|
| `dwErrorCode` | `DWORD` | Specifies the error code for the current thread. |

### Return Value

- **Type**: `VOID`
- None.

### Remarks

- Error codes are unsigned 32-bit values which are private to each calling thread. Most functions will set the last error code value when they fail; a few functions set it when they succeed. Function failure is typically indicated by a return value such as FALSE, NULL, INVALID_CLIENT or SMS_ERROR. Those functions which call SmsSetLastError when they succeed are noted on the function reference page.
- Applications can retrieve the value saved by this function by using the SmsGetLastError function. The use of SmsGetLastError is optional; an application can call the function to determine the specific reason for a function failure.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstxtv11.lib`

### See Also

- [SmsGetErrorString](#smsgeterrorstring-function)
- [SmsGetLastError](#smsgetlasterror-function)

---

## SmsUninitialize Function

Terminate use of the library by the application

### Prototype

```c
VOID WINAPI SmsUninitialize();
```

### Parameters

| Name | Type | Description |
|------|------|-------------|

### Return Value

- **Type**: `VOID`
- None.

### Remarks

- An application is required to perform a successful SmsInitialize call before it can call any of the other the library functions. When it has completed the use of library, the application must call SmsUninitialize to allow the library to free any resources allocated on behalf of the process. Any pending blocking or asynchronous calls in this process are canceled without posting any notification messages, and all sockets that were opened by the process are closed.
- There must be a call to SmsUninitialize for every successful call to SmsInitialize made by a process. In a multithreaded environment, operations for all threads in the client are terminated.

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Library: `cstxtv11.lib`

### See Also

- [SmsInitialize](#smsinitialize-function)

---

## SMSGATEWAY Structure

This structure contains information about a text message gateway server and the recipient.

### Definition

```c
typedef struct _SMSGATEWAY
{
    INT nGatewayId;
    INT nCountryCode;
    INT nAreaCode;
    INT nExchange;
    INT nMessageLength;
    DWORD dwReserved;
} SMSGATEWAY, *LPSMSGATEWAY;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `nGatewayId` | `INT` | An integer value which identifies the gateway record. This value is used internally and an application should not depend on the value not changing over time for a specific telephone number. As new area codes are introduced, the provider database will be updated to reflect these changes and that can result in a change to the gateway ID associated with a specific telephone number. |
| `nCountryCode` | `INT` | An integer value which specifies the ITU country calling code associated with the service provider. Currently this value will always be 1, which is the country code used by North American service providers. If the service provider database is expanded to include additional countries in the future, this value will identify the country of origin. |
| `nAreaCode` | `INT` | An integer value which specifies the Numbering Plan Area (NPA) code, commonly known as the area code. For the United States and Canada, area codes are assigned by the North American Numbering Plan Administration (NANPA). In North America, the area code is digits 1-3 for a 10-digit telephone number. This value, along with the exchange, is used to determine which company provides wireless service for a specific telephone number. |
| `nExchange` | `INT` | An integer value which specifies the exchange area. In North America, the exchange is digits 4-6 for a 10-digit telephone number. This value, along with the area code, is used to determine which company provides wireless service for a specific telephone number. |
| `nMessageLength` | `INT` | An integer value which specifies the maximum number of characters that the service provider will accept for a single text message. If the message exceeds this number of characters, the service provider may reject the message, or it may split the message into multiple messages. |
| `dwReserved` | `DWORD` | A value reserved for internal use. |

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`

---

## SMSMESSAGE Structure

This structure provides information about a text message.

### Definition

```c
typedef struct _SMSMESSAGE
{
    DWORD dwFormat;
    DWORD dwLength;
    DWORD dwFlags;
    DWORD dwReserved;
    LPCTSTR lpszProvider;
    LPCTSTR lpszPhoneNumber;
    LPCTSTR lpszSender;
    LPCTSTR lpszMessage;
} SMSMESSAGE, *LPSMSMESSAGE;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `dwFormat` | `DWORD` | An integer value which specifies the format of the message message. This member is included for future use where a service provider supports multiple message formats based on different versions of the protocol. The default value for this member is SMS_FORMAT_TEXT. |
| `dwLength` | `DWORD` | An integer value which specifies the length of the message. If this member is zero, the length will be automatically calculated based on the length of the lpszMessage text that is terminated by a null character. If this value is larger than the actual length of the message text, it will be ignored. |
| `dwFlags` | `DWORD` | An integer value which specifies one or more message options. |
| `dwReserved` | `DWORD` | Reserved for future use. This value should always be zero. |
| `lpszProvider` | `LPCTSTR` | A pointer to a null terminated string which specifies the name of the preferred wireless service provider responsible for handling the message. If this member is NULL or an empty string, the default provider assigned to the recipient's phone number will be used. This structure member is only used with SMS_SERVICE_SMTP messages and is ignored for other message services. |
| `lpszPhoneNumber` | `LPCTSTR` | A pointer to a null terminated string which specifies the recipient's phone number. This can be a standard E.164 formatted number or an unformatted number. Any extraneous whitespace, punctuation or other non-numeric characters in the string will be ignored. This structure member cannot be NULL. |
| `lpszSender` | `LPCTSTR` | A pointer to a null terminated string which identifies the sender of the message. For SMS_SERVICE_SMTP messages, this string should be a valid email address. For other services, this string may specify a phone number or shortcode. This structure member cannot be NULL. |
| `lpszMessage` | `LPCTSTR` | A pointer to a null terminated string that contains the message to be sent to the recipient. In most cases, a message should not exceed 160 characters in length, although some service providers may accept longer messages. If a message exceeds the maximum number of characters accepted by a service provider, the message may be ignored or it may be split into multiple messages. |

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SmsSendMessage](#smssendmessage-structure)
- [SMSSERVICE](#smsservice-structure)

---

## SMSPROVIDER Structure

This structure contains information about a wireless service provider.

### Definition

```c
typedef struct _SMSPROVIDER
{
    INT nProviderId;
    INT nCountryCode;
    INT nRegionCode;
    INT nMessageLength;
    DWORD dwFlags;
    DWORD dwReserved;
} SMSPROVIDER, *LPSMSPROVIDER;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `nProviderId` | `INT` | An integer value which identifies the provider record. This value is used internally and an application should not depend on the value not changing over time for a specific service provider. To uniquely identify a provider, use the szGuid member of the structure, which is guaranteed not to change as providers are added and removed from the database. |
| `nCountryCode` | `INT` | An integer value which specifies the ITU country calling code associated with the service provider. Currently this value will always be 1, which is the country code used by North American service providers. If the service provider database is expanded to include additional countries in the future, this value will identify the country of origin. |
| `nRegionCode` | `INT` | An integer value which identifies the region that the service provider covers. In North America, each region consists of multiple states and/or provinces. If a provider services multiple regions, this will identify the primary region where they provide coverage. |
| `nMessageLength` | `INT` | An integer value which specifies the maximum number of characters that the service provider will accept for a single text message. If the message exceeds this number of characters, the service provider may reject the message, or it may split the message into multiple messages. |
| `dwFlags` | `DWORD` | An unsigned integer value which specifies one or more flags that provides additional information about the service provider. This value is constructed by using a bitwise operator with any of the following constants: |
| `dwReserved` | `DWORD` | A value reserved for internal use. |

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`

---

## SMSSERVICE Structure

This structure provides information about the service used to send a text message.

### Definition

```c
typedef struct _SMSSERVICE
{
    DWORD dwServiceType;
    DWORD dwAuthType;
    DWORD dwVersion;
    DWORD dwTimeout;
    DWORD dwOptions;
    DWORD dwReserved;
    LPCTSTR lpszResource;
    LPCTSTR lpszAccount;
    LPCTSTR lpszUserName;
    LPCTSTR lpszPassword;
} SMSSERVICE, *LPSMSSERVICE;
```

### Members

| Name | Type | Description |
|------|------|-------------|
| `dwServiceType` | `DWORD` | An integer value which identifies the type of service that will be used to send the message. This member can be one of the following values: |
| `dwAuthType` | `DWORD` | An integer value which identifies the type of authentication used with the service. This member can be one of the following values: |
| `dwVersion` | `DWORD` | An integer value which identifies the interface version for the service being used. This member is included for future use where a service may support multiple versions of their interface and should normally be set to the value SMS_VERSION_DEFAULT. |
| `dwTimeout` | `DWORD` | An integer value which specifies the amount of time in seconds that a function will wait for a response from the service. If this value is zero, a default timeout period of 20 seconds will be used. If the service does not respond within this time period, the function will fail. |
| `dwOptions` | `DWORD` | An integer value which specifies one or more options. |
| `dwReserved` | `DWORD` | Reserved for future use. This value should always be zero. |
| `lpszResource` | `LPCTSTR` | A pointer to a null terminated string that specifies a resource for the service. Typically this will be either a fully qualified domain name or a URL. For gateways using SMTP, this string should identify the mail server. An alternate port number can also be specified by appending it to the hostname, separated by a colon. For example, smtp.company.com:587 would connect to the server on port 587. If you are specifying an IPv6 address with an alternate port number, the address must be enclosed in brackets. For services where a domain name or resource URL is not required, this member will be ignored and can be NULL. |
| `lpszAccount` | `LPCTSTR` | A pointer to a null terminated string that specifies an account name or identifier. Some service providers may require a unique account name or a token (application ID) in conjunction with other credentials. This member is not used with mail gateways and ignored if the service type is SMS_SERVICE_SMTP. If no account name is required for session authentication, this member can be NULL. |
| `lpszUserName` | `LPCTSTR` | A pointer to a null terminated string that specifies a user name to authenticate the session. If the authentication type is SMS_AUTH_USERNAME this member must specify a valid user name. If no authentication is required, this member may be NULL. Note that some service providers may use terminology other than "username" with their documentation; this member will always specify the first of a pair of authentication tokens. |
| `lpszPassword` | `LPCTSTR` | A pointer to a null terminated string that specifies the password used to authenticate the session. If the authentication type is SMS_AUTH_USERNAME this member must specify a valid password. If no authentication is required, this member may be NULL. Note that some service providers may use terminology other than "password" with their documentation; this member will always specify the second of a pair of authentication tokens. |

### Requirements

- Minimum Desktop Platform: Windows 7 Service Pack 1
- Minimum Server Platform: Windows Server 2008 R2 Service Pack 1
- Header: `cstools11.h`
- Unicode: Implemented as Unicode and ANSI versions.

### See Also

- [SmsSendMessage](#smssendmessage-structure)
- [SMSMESSAGE](#smsmessage-structure)

